<?php
Namespace App\Model\Admin;

use Symfony\Component\Translation\TranslatorInterface;
Use App\Utils\Utils;

/**
 * Provides admin functions for Homegear update management
 *
 */
class HomegearUpdate
{
    public $hg;
    private $utils;
    private $translator;
    private $componentUpdates = 'Updates';

    public function __construct(TranslatorInterface $translator, Utils $utils)
    {
        $this->hg = new \Homegear\Homegear;
        $this->utils = $utils;
        $this->translator = $translator;
    }

    public function getLastUpdateCheck()
    {
        return $this->hg->getData($this->componentUpdates,'lastUpdateCheck');
    }

    /**
     * Manually check for updates
     * 
     * @return array()
     */
    public function check()
    {
        if ($this->utils->checkManagementRunning(false)) {
            $this->hg->setData($this->componentUpdates,'lastUpdateCheck',time());
            $updateAvailable = false;
            $id = $this->hg->managementAptUpdate();
            $result = $this->utils->getCommandStatus($id);
            if ($result['exitCode'] == 0) {
                $updates['System'] = $this->hg->managementSystemUpdateAvailable(); 
                $updates['Homegear'] = $this->hg->managementHomegearUpdateAvailable();
                foreach ($updates as $type => $value) {
                    if ($value && $value != '') {
                        $this->hg->setData($this->componentUpdates,'UpdateAvailable_'.$type,true);
                        $updateAvailable = true;
                    } else {
                        $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_'.$type);
                    }
                }
                $result = exec('apt list --upgradable 2>/dev/null | grep -c homegear-smarthaui');
                if (isset($result) && intval($result) != 0) {
                    $this->hg->setData($this->componentUpdates,'UpdateAvailable_UI',true);
                    $updateAvailable = true;
                } else {
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_UI');
                }
                $result1 = exec('apt list --upgradable 2>/dev/null | grep -c homegear-smartha-devicedefinitions');
                if (isset($result1) && intval($result1) != 0) {
                    $this->hg->setData($this->componentUpdates,'UpdateAvailable_DeviceDefinitions',true);
                    $updateAvailable = true;
                } else {
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_DeviceDefinitions');
                }
                $result2 = exec('apt list --upgradable 2>/dev/null | grep -c homegear-smartha-nodes');
                if (isset($result2) && intval($result2) != 0) {
                    $this->hg->setData($this->componentUpdates,'UpdateAvailable_Nodes',true);
                    $updateAvailable = true;
                } else {
                    $this->hg->deleteData($this->componentUpdates,'UpdateAvailable_Nodes');
                }
                return ['result' => 'success', 'msg' => (!$updateAvailable ? $this->translator->trans("updates.notfound",array(),'admin') : $this->translator->trans("updates.found",array(),'admin') )];
            }
            return ['result' => 'error', 'msg' => $this->translator->trans('updates.error_checking',array(),'admin')];
        }
        return ['result' => 'error', 'msg' => $this->translator->trans('management.not_running',array(),'admin')];
    }

    /**
     * List all current updates
     * 
     * @return array
     */
    public function list()
    {
        // check if an update is running
        if ($this->utils->checkUpdateRunning('_sys-update-running') === true || $this->utils->checkUpdateRunning('_os-update-running') === true) {
            $os = -1;
            $system = -1;
            $deviceDefinition = -1;
            $ui = -1;
            $nodes = -1;
        } else {
            $os = $this->hg->getData($this->componentUpdates,'UpdateAvailable_System');
            $system = $this->hg->getData($this->componentUpdates,'UpdateAvailable_Homegear');
            $ui = $this->hg->getData($this->componentUpdates,'UpdateAvailable_UI');
            $deviceDefinition = $this->hg->getData($this->componentUpdates,'UpdateAvailable_DeviceDefinitions');
            $nodes = $this->hg->getData($this->componentUpdates,'UpdateAvailable_Nodes');
        }
        return ['system' => $system, 'os' => $os, 'device_definition' => $deviceDefinition, 'ui' => $ui, 'nodes' => $nodes];
    }

    /**
     * Update packages
     * 
     * @return json
     */
    public function update($type = null)
    {
        if ($type === null) { return ['result' => 'error', 'msg' => 'Update-type missing']; }
        if ($type == 'ui') {
            if ($this->utils->checkManagementRunning(false)) {
                // check if we have homegear updates, too (since we do not have a function to update the ui seperately)
                $id = $this->hg->managementAptUpgrade(intval(1));
                if ($this->hg->getData($this->componentUpdates,'UpdateAvailable_Homegear')) {
                    $type = 'sys';
                }
                $this->utils->setOled("Update in progress...");
                return ['result' => 'success', 'type' => $type, 'id' => $id];
            }
        } elseif ($type == 'device_definition') {
            if ($this->utils->checkManagementRunning(false)) {
                // check if we have homegear updates, too (since we do not have a function to update the definitions seperately)
                $id = $this->hg->managementAptUpgrade(intval(1));
                if ($this->hg->getData($this->componentUpdates,'UpdateAvailable_Homegear')) {
                    $type = 'sys';
                }
                $this->utils->setOled("Update in progress...");
                return ['result' => 'success', 'type' => $type, 'id' => $id];
            }
        } elseif ($type == 'nodes') {
            if ($this->utils->checkManagementRunning(false)) {
                // check if we have homegear updates, too (since we do not have a function to update the definitions seperately)
                $id = $this->hg->managementAptUpgrade(intval(1));
                if ($this->hg->getData($this->componentUpdates,'UpdateAvailable_Homegear')) {
                    $type = 'sys';
                }
                $this->utils->setOled("Update in progress...");
                return ['result' => 'success', 'type' => $type, 'id' => $id];
            }
        } elseif ($type == 'sys') {
            if ($this->utils->checkManagementRunning(false)) {
                $id = $this->hg->managementAptUpgrade(intval(1));
                $this->utils->setOled("Update in progress...");
                return ['result' => 'success', 'type' => $type, 'id' => $id];
            }
        } elseif ($type == 'os') {
            if ($this->utils->checkManagementRunning(false)) {
                $id = $this->hg->managementAptUpgrade(intval(0));
                //$this->utils->saveToSession(['_os-update-running' => $id]);
                $this->hg->setData($this->componentUpdates,'UpdateRunning',$id);
                $this->utils->setOled("Update in progress...");
                return ['result' => 'success', 'type' => $type, 'id' => $id];
            }
        }
        return ['result' => 'error', 'msg' => 'Error getting update'];
    }
}